## ######### PROJECT NAME : ##########
##
## lsi_arrays.pm for Lib_Raid
##
## ######### PROJECT DESCRIPTION : ###
##
## This file is a part of cl.pm
##
## ###################################
##
## Made by Emmanuel Florac
## Email   <dev@intellique.com>
##
## Started on  Tue Mar 24 17:44:28 2009 Boutonnet Alexandre
##
## ###################################
##
use strict;
use warnings;
use Data::Dumper;

our $CONTROLLER_PREFIX;
our $lsi_cmd;

# This function returns an hash containing
# all arrays informations
# It takes the controller name in parameter
# This function returns a array with :
# [0] : 0 => Ok, != O => Fail
# [1] : Ok => hash, Fail => error_msg
sub get_arrays_info {
    my $controller = shift;

    my $hash = {};

    my ($controller_name) = ( $controller =~ m/$CONTROLLER_PREFIX(\d+)/ );
    return ( 1, "unable to find the controller $controller" )
      if ( !defined($controller_name) );

    # getting informations about drives
    my $cmd = "$lsi_cmd /c$controller_name/vall show all";
    my ( $ret_code, $data ) = _exec_cmd($cmd);
    return ( $ret_code, "unable to get array informations : $data" )
      if ($ret_code);

    my %factor = ( MB => 1, GB => 1024, TB => 1048576 );

    # splitting my output string in an array
    my @tmp_tab = split( /\n/, $data );

    my $current_array = -1;
    my $currentdrive  = 0;

    foreach my $line (@tmp_tab) {
        $current_array = $1
          if ( $line =~ m(/c$controller_name/v(\d+) :) );

        $hash->{"a$current_array"}{stripesize} = $1
          if ( $line =~ m(Strip Size = (\d+)) );

        $hash->{"a$current_array"}{blocks} = $1
          if ( $line =~ m(Number of Blocks = (\d+)) );

        if ( $line =~
m(^\d+\/\d+\s+(\w+)\s+(\w+)\s+\w+\s+(\w+)\s+(\w+)\s+\-\s+\w+\s+(\d+\.\d+)\s+(\w+)\s+(\w+))
          )
        {
            $hash->{"a$current_array"}{raidtype} =
              lib_raid_codes::get_raid_level_code($1);
            $hash->{"a$current_array"}{status} = $3 eq 'Yes' ? 0 : 2;
            $hash->{"a$current_array"}{status} =
              lib_raid_codes::get_state_code($2);
            $hash->{"a$current_array"}{cachemode} = $4;
            my $unit = $6;
            $hash->{"a$current_array"}{size} = $5 * $factor{$unit};
            $hash->{"a$current_array"}{name} = $7;
            next;
        }

        if ( $line =~
            m(^(\d+):(\d+)\s+\d+\s+(\w+)\s+($current_array)\s+\d+\.\d*\s+\wB) )
        {
            push @{ $hash->{"a$current_array"}{drives} }, "d$currentdrive";
            my $drive_status = lib_raid_codes::get_drive_status_code($3);
            if ( $drive_status == 2 ) {
                (
                    $hash->{"a$current_array"}{progression},
                    $hash->{"a$current_array"}{status}
                ) = _get_drive_rebuild( $controller_name, $1, $2 );
            }

            $currentdrive++;
            next;
        }

        if ( $line =~ m(Active Operations = ([\w\s]+)\((\d+)%\)) ) {
            $hash->{"a$current_array"}{status} =
              lib_raid_codes::get_state_code($1);
            $hash->{"a$current_array"}{progression} = $2;
        }

    }
	
    #	die Dumper $hash;
    return ( 0, $hash );
}

sub _get_drive_rebuild {
	my ($ctl, $enc, $slot) = @_;

    my $cmd = "$lsi_cmd /c$ctl/e$enc/s$slot show rebuild";
    my ( $ret_code, $data ) = _exec_cmd($cmd);
    return ( $ret_code, "unable to get drive rebuild progress : $data" )
      if ($ret_code);
	
	my @reb_data = split(/\n/, $data);
	foreach my $line (@reb_data) {
		if ( $line =~ m(^/c$ctl/e$enc/s$slot\s+(\d+|-)\s+(In progress|Paused)) ) {
			if ( $2 eq 'In progress') {
				return($1, 5);
			} else {
				return(-1, 10);
			}
		}
	}
	
	return(undef,undef);	
}

1;
