## ######### PROJECT NAME : ##########
##
## lvm_drives.pm for Lib_Raid
##
## ######### PROJECT DESCRIPTION : ###
##
## This file is a part of lvm.pm
##
## ###################################
##
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
##
## Started on  Wed Mar 11 11:30:55 2009 Boutonnet Alexandre
## Last update Mon Mar 23 17:57:24 2009 Boutonnet Alexandre
##
## ###################################
##
use strict;
use warnings;

our $CONTROLLER_PREFIX;

# This function returns an hash containing
# all drives informations
# It takes the controller name in parameter
# This function returns a array with :
# [0] : 0 => Ok, != O => Fail
# [1] : Ok => hash, Fail => error_msg
sub get_drives_info {
    my $controller = shift;

    my $hash = {};

    my ( $ret_code, $data ) = get_drives_list($controller);
    return ( $ret_code, $data ) if ($ret_code);

    foreach my $drive (@$data) {
        ( $ret_code, $data ) = get_drive_info( $controller, $drive );
        return ( $ret_code, $data ) if ($ret_code);
        $hash->{$drive} = $data;
    }
    return ( 0, $hash );
}

# This function returns an hash containing
# all informations about the drive given in
# parameter. Controller name had to be given too.
# [0] : 0 => Ok, != O => Fail
# [1] : Ok => hash, Fail => error_msg
sub get_drive_info {
    my ( $controller, $drive ) = @_;
    my $hash = {
        'status'          => 0,
        'enclosurenumber' => -1,
        'model'           => 'N/A',
        'slotnumber'      => '-1',
        'type'            => -1,
        'connectornumber' => -1,
        'vendor'          => 'N/A',
    };

    return ( 1, "unable to find the controller $controller" )
      if ( $controller !~ m/$CONTROLLER_PREFIX/ );

    return ( 1, 'unable to get drive_number' ) if ( $drive !~ m/\/\w+\/\w+/ );

    # getting informations about the drive
    my $cmd = "pvdisplay -c $drive";
    my ( $ret_code, $data ) = _exec_cmd($cmd);

    # TODO
    #     return (_get_failed_drive_info($controller, $drive)) if ($ret_code);
    my @drive_tmp_tab = split( /\n/, $data );
    if ( $drive_tmp_tab[0] =~ m/new physical volume/ ) {
        $data = $drive_tmp_tab[1];
    }
	
	chomp $data;
	
    (
        undef, $hash->{inarray}, $hash->{size}, undef, undef, undef, undef,
        undef, undef, undef, undef, $hash->{serial}
    ) = split( /:/, $data );

    $hash->{inarray} = lib_raid_codes::get_drive_inarray_code("unused")
      if not $hash->{inarray};
	  
    # model
    # TODO should check for symlinks, etc
    my ($base_device) = ( $drive =~ m#^/dev/([a-z]+)\d*# );

	# size is given in sectors...
	my $sectorsize = 512;
	if (open my $fss, '<', "/sys/block/$base_device/queue/hw_sector_size") {
		$sectorsize = <$fss>;
	}
	
	$hash->{size} /= ( 1024 * 1024 / $sectorsize );
	
    my %tmphash;
    for my $prop (qw( vendor model )) {
        open my $fv, '<', "/sys/block/$base_device/device/$prop"
          or next;
        $tmphash{$prop} = <$fv>;
        chomp $tmphash{$prop};
        $tmphash{$prop} =~ s/\s+//g;
        close $fv;
    }

    $hash->{vendor} = $tmphash{vendor} if $tmphash{vendor};
    $hash->{model}  = $tmphash{model}  if $tmphash{model};

    return ( 0, $hash );
}

# This function returns the drive list
# It takes the controller name in parameter
# [0] : 0 => Ok, != O => Fail
# [1] : Ok => array, Fail => error_msg
sub get_drives_list {
    my $controller = shift;

    return ( 1, "unable to find the controller $controller" )
      if ( $controller !~ m/$CONTROLLER_PREFIX/ );

    # getting informations about drives
    my $cmd = 'pvdisplay -c';
    my ( $ret_code, $data ) = _exec_cmd($cmd);
    return ( $ret_code, "unable to get drives informations : $data" )
      if ($ret_code);

    # splitting my output string in an array
    my @tmp_tab = split( /\n/, $data );

    my $drives_list = ();
    foreach my $line (@tmp_tab) {
        my ($drive) = ( $line =~ m/^ +([\w\/]+):/ );
        push( @$drives_list, $drive ) if $drive;
    }
	
    return ( 0, $drives_list );
}

1;
